%convert the match info outputted from PatchMatching.m to another 
%data stucture for convenient use in the next step
%Input:
%   matchresult --- match info computed in PatchMatching.m
%   cfno --- the current frame number 
%   blksize --- block size (centered) 
%   overlapsize --- comparison is done for block of size(blkszie+2*overlapsize)
%Output:
%   matchinfo --- a cell structure; each cell records match info of one block
%                   .fn --- current frame number
%                   .y  ---- upperleft corner of one block
%                   .x  ---- upperleft corner of one block
%                   .psize (=w=h)
%                   .matchdata{1},{2},{3},....
%                   matchdata{i}.fn -- frame number of matching block
%                   matchdata{i}.y -- upperleft corner of matching block
%                   matchdata{i}.x -- upperleft corner of matching block        
        
function matchinfo = MatchInfo(matchresult, cfno, blksize, overlapsize, maxw, maxh)

disp = {[0 0]};
disp = {[0 0], [0 0], [0 0]};
disp = {[0 0],...
        [1 0],...
        [0 1],...
        [-1 0],...
        [0 -1]};

nf = length(matchresult);
pno = length(matchresult{1});
mw = maxw-(blksize+2*overlapsize)+1;
mh = maxh-(blksize+2*overlapsize)+1;
for i=1:pno
    w1 = matchresult{cfno}{i}.y;
    h1 = matchresult{cfno}{i}.x;
    lt = w1-overlapsize;
    rt = w1+blksize+overlapsize-1;
    lb = h1-overlapsize;
    rb = h1+blksize+overlapsize-1;
    [uly, ulx] = boundary_check(lt, rt, lb, rb, maxw, maxh, mw, mh);
    matchinfo{i}.fn = cfno;
    matchinfo{i}.y = uly;
    matchinfo{i}.x = ulx;
    matchinfo{i}.psize = blksize+2*overlapsize;
    matchinfo{i}.matchdata = zeros(4,1+nf*length(disp));
    %always put the first matching patch to be the patch itself 
    matchinfo{i}.matchdata(1,1) = cfno;
    matchinfo{i}.matchdata(2,1) = uly;
    matchinfo{i}.matchdata(3,1) = ulx;
    matchinfo{i}.matchdata(4,1) = 1;
    mno = 2;
    %include in some neighboring patches
    for nn = 1:length(disp)
        dy = disp{nn}(1); dx = disp{nn}(2); 
        [uly, ulx] = boundary_check(lt+dy, rt+dy, lb+dx, rb+dx, maxw, maxh, mw, mh);
        matchinfo{i}.matchdata(1,mno) = cfno;
        matchinfo{i}.matchdata(2,mno) = uly;
        matchinfo{i}.matchdata(3,mno) = ulx;
        if dy==0 && dx==0
            matchinfo{i}.matchdata(4,mno) = 1;
        else
            matchinfo{i}.matchdata(4,mno) = 1;
        end
        mno = mno+1;
    end
    for j=1:nf
        if j ~= cfno
            w1 = matchresult{j}{i}.matchdata{1}.y;
            h1 = matchresult{j}{i}.matchdata{1}.x;
            lt = w1-overlapsize;
            rt = w1+blksize+overlapsize-1;
            lb = h1-overlapsize;
            rb = h1+blksize+overlapsize-1;
            %include in some neighboring patches
            for nn = 1:length(disp)
                dy = disp{nn}(1); dx = disp{nn}(2); 
                [uly, ulx] = boundary_check(lt+dy, rt+dy, lb+dx, rb+dx, maxw, maxh, mw, mh);
                matchinfo{i}.matchdata(1,mno) = j;
                matchinfo{i}.matchdata(2,mno) = uly;
                matchinfo{i}.matchdata(3,mno) = ulx;
                if dy==0 && dx==0
                    matchinfo{i}.matchdata(4,mno) = 1;
                else
                    matchinfo{i}.matchdata(4,mno) = 1;
                end
                mno = mno+1;
            end
        end
    end
end
%=======================================================================
%check boundary of the matching block
function [uly, ulx] = boundary_check(lt, rt, lb, rb, maxw, maxh, mw, mh)

if lt>=1 && lb>=1 && rt<=maxw && rb<=maxh
    uly = lt; ulx = lb;
end
if lt<=0 && (lb>=1 && rb<=maxh)
    uly = 1; ulx = lb;
end
if (lt>=1 && rt<=maxw) && lb<=0
    uly = lt; ulx = 1;
end
if rt>maxw && (lb>=1 && rb<=maxh)
    uly = mw; ulx = lb;
end 
if (lt>=1 && rt<=maxw) && rb>maxh
    uly = lt; ulx = mh;
end
if lt<=0 && lb<=0
    uly = 1; ulx = 1;
end
if lt<=0 && rb>maxh
    uly = 1; ulx = mh;
end
if rt>maxw && lb<=0
    uly = mw; ulx = 1;
end 
if rt>maxw && rb>maxh
    uly = mw; ulx = mh;
end
%=======================================================================