function u = PatchDenoiseSL(vData, matchinfo, sigma)
%Patch-based image denoise using robust PCA with missing data techinque
%INPUT:
    %vData -- Noisy image sequence
    %matchinfo -- the patch matching result
    %datasite --- 1 for trustable data sites, 0 for others
    %sigma --- standard deviation 
%OUTPUT:
    %u -- the denoised image 
%image size 
[m,n,co] = size(vData(1).cdata);

%perform matrix completion on patch stacks
pno = length(matchinfo);
ph = matchinfo{1}.psize;
mpno = size(matchinfo{1}.matchdata, 2);
patchdata = zeros(ph^2*co, mpno);
psite = zeros(ph^2*co, mpno);
f = zeros(m,n,co);
%Trust data,1 for reliable,0 for unreliable;
datasite = ones([size(vData(1).cdata) length(vData)],'uint8');
% profile on;
for i=1:pno
%for i=3000:4000
    
    for j=1:mpno
        tfn = matchinfo{i}.matchdata(1,j);
        ty = matchinfo{i}.matchdata(2,j);
        tx = matchinfo{i}.matchdata(3,j);
        wt = matchinfo{i}.matchdata(4,j);
        tmp = wt*double(vData(tfn).cdata(ty:ty+ph-1,tx:tx+ph-1,:));
        patchdata(:,j) = tmp(:);
        tmp = double(datasite(ty:ty+ph-1,tx:tx+ph-1,:,tfn));
        psite(:,j) = tmp(:);
    end
    [L_hat,S_hat] = rPCA(patchdata,psite,sigma);
%     profile viewer;
    %following we assume the image is on the first column
    ty = matchinfo{i}.y;
    tx = matchinfo{i}.x;    
    f(ty:ty+ph-1,tx:tx+ph-1,:) = f(ty:ty+ph-1,tx:tx+ph-1,:)+reshape(L_hat(:,1),ph,ph,co);
%     display(i);
end
% profsave(p,'profile_results');
%weighted output denoise image
D = zeros(m,n,co);
for i=1:pno
    ty = matchinfo{i}.y;
    tx = matchinfo{i}.x;            
    D(ty:ty+ph-1,tx:tx+ph-1,:) = D(ty:ty+ph-1,tx:tx+ph-1,:)+ones(ph,ph,co);
end
u = f./D;

%robust PCA technique to separate two layers: low-rank and sparse part
function [L_hat,S_hat] = rPCA(patchdata,psite,sigma)

[nr,nc] = size(patchdata);
p = nnz(psite)/(nr*nc);
%lambda = 1*1/sqrt(max(nr,nc));
lambda = 0.8*1/sqrt(max(nr,nc));
% mu = 1*sqrt(p)*sigma*(sqrt(nr)+sqrt(nc));
mu = 2*(sqrt(nc)+sqrt(nc))*sigma;
maxIter = 30;
tol = 1e-2;
[L_hat,S_hat,iter] = partial_proximal_gradient_rpca2(patchdata, psite, lambda, mu, maxIter, tol);
%===========================================================================