%patch matching between image sequence by a simple search method
%Input:
%   imgsData --- image sequence read from an avi file
%   k --- k-th frame to be matched
%   blksize --- block size (centered) 
%   searchsize --- search size between two adjacent frame
%   overlapsize --- comparison is done for block of size(blkszie+2*overlapsize)
%Output:
%   matchresult --- a cell structure; each cell records match info of one block
%                   .y
%                   .x
%                   .w
%                   .h
%                   .matchdata{1},{2},{3},....
%                   matchdata{i}.y
%                   matchdata{i}.x
%                   matchdata{i}.err

function match_info = PatchMatching(imgsData, k, blksize, searchsize, overlapsize)


% matching parameter
    blksize=4;  %patchsize
    searchsize=6;
    overlapsize=2;
    normdim=1;
    im0 = double(imgsData(k).cdata);
    tic;
    disp('step 1');
    matchresult = cell(1, length(imgsData));
    [matchdataI0] = MatchIntraframe(im0, blksize, searchsize, overlapsize, normdim);
%    displayImgMatchResult(matchdataI);
    matchresult{k} = matchdataI0;
    matchdataI = matchdataI0;   
    for i = k-1:-1:1
        ss = sprintf('step %d', i);
        disp(ss);
        im1 = double(imgsData(i).cdata);
        [matchdataI] = MatchInterframe(im0, im1, matchdataI, searchsize, overlapsize, normdim);
        displayImgMatchResult(matchdataI);
        matchresult{i} = matchdataI;
    end
    matchdataI = matchdataI0;
    for i = k+1:length(imgsData);
        ss = sprintf('step %d', i);
        disp(ss);
        im1 = double(imgsData(i).cdata);
        [matchdataI] = MatchInterframe(im0, im1, matchdataI, searchsize, overlapsize, normdim);
        displayImgMatchResult(matchdataI);
        matchresult{i} = matchdataI;
    end
    toc;
      maxw = size(imgsData(1).cdata,1);
      maxh = size(imgsData(1).cdata,2);
      match_info = MatchInfo(matchresult, k, blksize, overlapsize, maxw, maxh);
return;


function [paraout] = MatchInterframe(im0, im1, parain, searchsize, overlapsize, normdim)
    searchblknumber=1;
    stepsize = 2;
    
    [h, w, co] = size(im0);
    paraout = parain;
    for i = 1:length(parain)
        ROI = parain{i};
        [imblk] = getPatch(im0, ROI, overlapsize);
        if length(imblk) > 0 %otherwise it is same as current blk
            data = ROI.matchdata;
            for j = 1:length(data)
                blkdata = data{j};
%                area.y = ROI.y; area.x = ROI.x;   %based on current frame
                area.y = blkdata.y; area.x = blkdata.x; %based on neighbour frame
                area.h = ROI.h; area.w = ROI.w;
                
                if i==400
                    [rdata] = matchblock(imblk, im1, normdim, searchblknumber, stepsize, area, searchsize, overlapsize);
                else
                    [rdata] = matchblock(imblk, im1, normdim, searchblknumber, stepsize, area, searchsize, overlapsize);
                end
           
                data{j} = rdata{1};
            end
            ROI.matchdata = data;
        else
            [imblk] = getPatch(im0, ROI, 0);
            if length(imblk) > 0 %otherwise it is same as current blk
                data = ROI.matchdata;
                for j = 1:length(data)
                    blkdata = data{j};
%                area.y = ROI.y; area.x = ROI.x;   %based on current frame
                    area.y = blkdata.y; area.x = blkdata.x; %based on neighbour frame
                    area.h = ROI.h; area.w = ROI.w;
        
                    [rdata] = matchblock(imblk, im1, normdim, searchblknumber, stepsize, area, searchsize, 0);
           
                    data{j} = rdata{1};
                end
                ROI.matchdata = data;
            else
                ROI.matchdata{1}.err = 999;
            end
        end
        paraout{i} = ROI;
    end
return;


function [result] = MatchIntraframe(im0, blksize, searchsize, overlapsize, normdim)
    
    [h, w, co] = size(im0);
    index = 1; result = [];
%    im1 = im0;
    for y0 = 1:blksize:h
        for x0 = 1:blksize:w
%            imblk = im0(y0:y0+blksize-1, x0:x0+blksize-1,:);
            
%            area.h = h-8; area.w = w-8;
%            area.y = 1;  area.x = 1;
%%            
%            [data] = matchblock(imblk, im1, normdim, 5, area, 3);     
            blkdata.y =  y0; blkdata.x =  x0; blkdata.err = 0; %do nothing
            data{1} = blkdata;
%%
            ROI.y = y0; ROI.x = x0;
            ROI.h = blksize;  ROI.w = blksize;
            ROI.matchdata = data;
            result{index} = ROI;
            index = index+1;
%            displayBlkMatchResult(ROI);
        end
    end
return;

function displayImgMatchResult(data)
    for i = 400:400%length(data);
        displayBlkMatchResult(data{i});
    end
return;

function displayBlkMatchResult(ROI)
    ss = sprintf('[y,x]:[%d,%d],[%d,%d]', ROI.y, ROI.x,  ROI.w, ROI.h);
    disp(ss);
    data = ROI.matchdata;
    for j = 1:length(data)
        mdata = data{j};
        ss = sprintf('Match: [%d,%d], Err:%d', mdata.y, mdata.x, mdata.err);
        disp(ss);
    end
return;

function [result] = matchblock(imblk, imwhole, normdim, M, stepsize, lefttoparea, searchsize, overlapsize)
    imblk = double(imblk);
    imwhole = double(imwhole);
    ROI = lefttoparea;    
    index = 0;
    for y = -searchsize:stepsize:searchsize
        for x = -searchsize:stepsize:searchsize
            ROI.y = lefttoparea.y+y;
            ROI.x = lefttoparea.x+x;
            [im] = getPatch(imwhole, ROI, overlapsize);
            if length(im) > 0
                index = index+1;
                py(index) = ROI.y;
                px(index) = ROI.x;
                det = im-imblk;
                %allerr(index) = mynorm(det,normdim);
                allerr(index) = sum(abs(det(:)));
            end
        end
    end
    
    [err, I] = sort(allerr);
    M = min(M, index);
    result = [];
    for i = 1:M
        data.y =  py(I(i));
        data.x =  px(I(i));
        data.err = err(i);
        result{i} = data;
    end
return;

function [imblk] = getPatch(im, ROI, overlapsize)
    [h, w, co] = size(im);
    y0 = ROI.y-overlapsize; y1 = ROI.y+overlapsize+ROI.h-1;
    x0 = ROI.x-overlapsize; x1 = ROI.x+overlapsize+ROI.w-1;
    if (y0 < 1 || x0 < 1 || y1 > h || x1 > w)
        imblk = [];
    else
        imblk = im(y0:y1, x0:x1,:);
    end
return;

function err = mynorm(det,normdim)
    [h,w,co]=size(det);
    if co > 1
        err = 0;
        for i = 1:co
            x = det(:,:,i);
            err1 = norm(x,normdim)/(w*h*co);
            err = err+err1;
        end
    else
            err = norm(det,normdim)/(w*h);
    end
return;

function [x] = intTruncate(x, w)
    if x < 1
        x = 1;
    end
    if x > w
        x = w;
    end
return;
